;-------------------------------------------------------------------------------
;
; Name        : RequesterPlus
; Version     : 0.2
;
; Author      : Flype ( flype@altern.org )
; Date        : 09 may 2004
;
; Purpose     : RequesterPlus provide a little bit more requesters
;
; Compatibily : Some commands are designed for Microsoft Windows only
; Compilation : TailBite 1.1 PR 10 and PureBasic 3.90
;
; Commands    : AboutShellRequester( Title.s, Text1.s, Text2.s, hIcon.l)
;
;               ListViewRequester(WindowID.l, Title.s, Message.s)
;               ListViewRequester_AddItem(Item.s)
;               ListViewRequester_Clear()
;
;               TextRequester(WindowID.l, Title.s, Message.s, Text.s)
;               TextBox(WindowID.l, Title.s, Message.s, Text.s)
;
;               InfoBox(Title.s, Message.s) 
;               WarningBox(Title.s, Message.s) 
;               ErrorBox(Title.s, Message.s) 
;               QuestionBox(Title.s, Message.s, Flags.l)
;
;-------------------------------------------------------------------------------




;-------------------------------------------------------------------------------
; Initialisation
;-------------------------------------------------------------------------------


#Req_Ok     = "OK"
#Req_Cancel = "Cancel"


Structure ListViewRequester_Struct
  
  hWnd.l
  
  MinWidth.l
  MinHeight.l
  
  iFrame.l
  iMessage.l
  iList.l
  iOK.l
  iCancel.l
  
EndStructure
Structure TextRequester_Struct
  
  hWnd.l
  
  MinWidth.l
  MinHeight.l
  
  iFrame.l
  iMessage.l
  iText.l
  iOK.l
  iCancel.l
  
EndStructure
Structure TextBox_Struct
  
  hWnd.l
  
  MinWidth.l
  MinHeight.l
  
  iFrame.l
  iMessage.l
  iText.l
  iOK.l
  
EndStructure


;-------------------------------------------------------------------------------
; Library Initialisation
;-------------------------------------------------------------------------------


ProcedureDLL RequesterPlus_Init()
  
  ; Font
  
  Global FontBold.l
  iFont.l = LoadFont(#PB_Any, "Arial", 9, #PB_Font_Bold)
  FontBold.l = FontID()
  
  ; ListViewRequester
  
  Global LVR.ListViewRequester_Struct
  NewList ListViewRequester_List.s()
  
  ; TextRequester
  
  Global TEXTREQ.TextRequester_Struct
  
  ; TextBox
  
  Global TEXTBOX.TextBox_Struct
  
EndProcedure


;-------------------------------------------------------------------------------
; Private commands
;-------------------------------------------------------------------------------


Procedure EditorGadgetWrap(WindowID.l,bool.b)
  
  If WindowID <> #Null
    If bool
      SendMessage_(WindowID, #EM_SETTARGETDEVICE, #Null, $000000)
    Else
      SendMessage_(WindowID, #EM_SETTARGETDEVICE, #Null, $FFFFFF)
    EndIf
  EndIf
  
EndProcedure
Procedure ProcessMinMaxInfo(WindowID.l, lParam.l, MinWidth.l, MinHeight.l)
  
  *pMinMax.MINMAXINFO = lParam
  *pMinMax\ptMinTrackSize\x = MinWidth
  *pMinMax\ptMinTrackSize\y = MinHeight
  *pMinMax\ptMaxTrackSize\x = GetSystemMetrics_(#SM_CXSCREEN)
  *pMinMax\ptMaxTrackSize\y = GetSystemMetrics_(#SM_CYSCREEN)
  
EndProcedure
Procedure ListViewRequester_CB(WindowID.l, Message.l, wParam.l, lParam.l)
  
  Result = #PB_ProcessPureBasicEvents
  
  If WindowID <> LVR\hWnd
    ProcedureReturn Result
  EndIf
  
  Select Message
      
    Case #WM_SIZE
      
      ww = WindowWidth()
      wh = WindowHeight()
      ResizeGadget(LVR\iFrame,-1,-1,ww-10,-1)
      ResizeGadget(LVR\iMessage,-1,-1,ww-20,-1)
      ResizeGadget(LVR\iList,-1,-1,ww-10,wh-90)
      ResizeGadget(LVR\iOK,-1,wh-32,-1,-1)
      ResizeGadget(LVR\iCancel,ww-105,wh-32,-1,-1)

    Case #WM_GETMINMAXINFO
      
      ProcessMinMaxInfo(WindowID, lParam, LVR\MinWidth, LVR\MinHeight)
      
  EndSelect
  
  ProcedureReturn Result
  
EndProcedure
Procedure TextRequester_CB(WindowID.l, Message.l, wParam.l, lParam.l)
  
  Result = #PB_ProcessPureBasicEvents
  
  If WindowID <> TEXTREQ\hWnd
    ProcedureReturn Result
  EndIf
  
  Select Message
    
    Case #WM_SIZE
      
      ww = WindowWidth()
      wh = WindowHeight()
      ResizeGadget(TEXTREQ\iFrame,-1,-1,ww-10,-1)
      ResizeGadget(TEXTREQ\iMessage,-1,-1,ww-20,-1)
      ResizeGadget(TEXTREQ\iText,-1,-1,ww-10,wh-90)
      ResizeGadget(TEXTREQ\iOK,-1,wh-32,-1,-1)
      ResizeGadget(TEXTREQ\iCancel,ww-105,wh-32,-1,-1)
      
    Case #WM_GETMINMAXINFO
      
      ProcessMinMaxInfo(WindowID, lParam, TEXTREQ\MinWidth, TEXTREQ\MinHeight)
      
  EndSelect
  
  ProcedureReturn Result
  
EndProcedure
Procedure TextBox_CB(WindowID.l, Message.l, wParam.l, lParam.l)
  
  Result = #PB_ProcessPureBasicEvents
  
  If WindowID <> TEXTBOX\hWnd
    ProcedureReturn Result
  EndIf
  
  Select Message
    
    Case #WM_SIZE
      
      ww = WindowWidth()
      wh = WindowHeight()
      ResizeGadget(TEXTBOX\iFrame,-1,-1,ww-10,-1)
      ResizeGadget(TEXTBOX\iMessage,-1,-1,ww-20,-1)
      ResizeGadget(TEXTBOX\iText,-1,-1,ww-10,wh-90)
      ResizeGadget(TEXTBOX\iOK,ww/2-GadgetWidth(TEXTBOX\iOK)/2,wh-32,-1,-1)
      
    Case #WM_GETMINMAXINFO
      
      ProcessMinMaxInfo(WindowID, lParam, TEXTBOX\MinWidth, TEXTBOX\MinHeight)
      
  EndSelect
  
  ProcedureReturn Result
  
EndProcedure


;-------------------------------------------------------------------------------
; Public commands : AboutShellRequester
;-------------------------------------------------------------------------------


ProcedureDLL.l AboutShellRequester(Title.s, Text1.s, Text2.s, hIcon.l)
  
  If Text1 <> ""
    Title + " # " + Text1
  EndIf
  
  ProcedureReturn ShellAbout_(#Null, Title, Text2, hIcon)
  
EndProcedure


;-------------------------------------------------------------------------------
; Public commands : MessageBox
;-------------------------------------------------------------------------------


ProcedureDLL.l InfoBox(Title.s, Message.s)
  ProcedureReturn MessageRequester(Title, Message, #MB_ICONINFORMATION)
EndProcedure
ProcedureDLL.l WarningBox(Title.s, Message.s)
  ProcedureReturn MessageRequester(Title, Message, #MB_ICONWARNING)
EndProcedure
ProcedureDLL.l ErrorBox(Title.s, Message.s)
  ProcedureReturn MessageRequester(Title, Message, #MB_ICONERROR)
EndProcedure
ProcedureDLL.l QuestionBox(Title.s, Message.s, Flags.l)
  ProcedureReturn MessageRequester(Title, Message, Flags|#MB_ICONQUESTION)
EndProcedure


;-------------------------------------------------------------------------------
; Public commands : ListViewRequester
;-------------------------------------------------------------------------------


ProcedureDLL.l ListViewRequester_AddItem(Item.s)
  
  If Item <> ""
    If AddElement(ListViewRequester_List())
      ListViewRequester_List() = Item
    EndIf
  EndIf

EndProcedure
ProcedureDLL.l ListViewRequester_Clear()
  
  ClearList(ListViewRequester_List())
  
EndProcedure
ProcedureDLL.l ListViewRequester(WindowID.l, Title.s, Message.s)
  
  Result = -1
  
  iWnd.l = OpenWindow(#PB_Any, 0, 0, 250, 300, #PB_Window_SizeGadget|#PB_Window_ScreenCentered, Title, WindowID)
  If iWnd = #Null
    ProcedureReturn Result
  EndIf

  LVR\hWnd = WindowID(iWnd)
    
  If CreateGadgetList(LVR\hWnd) = #Null
    ProcedureReturn Result
  EndIf
  
  LVR\MinWidth  = WindowWidth()
  LVR\MinHeight = WindowHeight()
  
  LVR\iFrame    = Frame3DGadget (#PB_Any,  5,  5,   0, 36, "")
  LVR\iMessage  = TextGadget    (#PB_Any, 10, 17,   0, 20, Message, #PB_Text_Center)
  LVR\iList     = ListViewGadget(#PB_Any,  5, 50,   0,  0)
  LVR\iOK       = ButtonGadget  (#PB_Any,  5,  0, 100, 25, #Req_Ok, #PB_Button_Default)
  LVR\iCancel   = ButtonGadget  (#PB_Any,  0,  0, 100, 25, #Req_Cancel)
  
  SetGadgetFont(LVR\iMessage,FontBold)
  SetGadgetFont(LVR\iOK,FontBold)
  SetGadgetFont(LVR\iCancel,FontBold)

  ForEach ListViewRequester_List()
    AddGadgetItem(LVR\iList, -1, ListViewRequester_List())
  Next
  
  DisableGadget(LVR\iOK,#True)
  SetWindowCallback(@ListViewRequester_CB())
  ResizeWindow(LVR\MinWidth,LVR\MinHeight-10)
  AddKeyboardShortcut(iWnd, #PB_Shortcut_Escape, 0) 
  AddKeyboardShortcut(iWnd, #PB_Shortcut_Return, 1) 
  
  Repeat
    
    Event = WaitWindowEvent()
    
    If EventWindowID() = iWnd
    
      Select Event
        
        Case #PB_Event_CloseWindow : Break
        
        Case #PB_Event_Menu
          
          Select EventMenuID()
            Case 0 : Break
            Case 1
              If GetGadgetState(LVR\iList) <> -1
                Result = GetGadgetState(LVR\iList) : Break
              EndIf
          EndSelect
          
        Case #PB_Event_Gadget
          
          Select EventGadgetID()
            Case LVR\iOK
              If GetGadgetState(LVR\iList) <> -1
                Result = GetGadgetState(LVR\iList) : Break
              EndIf
            Case LVR\iList
              DisableGadget(LVR\iOK,#False)
              If EventType()=#PB_EventType_LeftDoubleClick
                Result = GetGadgetState(LVR\iList) : Break
              EndIf
            Case LVR\iCancel : Break
          EndSelect
          
      EndSelect
      
    EndIf
    
  ForEver
  
  ClearGadgetItemList(LVR\iList)
  FreeGadget(LVR\iFrame)
  FreeGadget(LVR\iMessage)
  FreeGadget(LVR\iList)
  FreeGadget(LVR\iOK)
  FreeGadget(LVR\iCancel)
  CloseWindow(iWnd)
  
  ProcedureReturn Result
  
EndProcedure


;-------------------------------------------------------------------------------
; Public commands : TextBox & TextRequester
;-------------------------------------------------------------------------------


ProcedureDLL.l TextBox(WindowID.l, Title.s, Message.s, Text.s)
  
  iWnd.l = OpenWindow(#PB_Any, 0, 0, 250, 300, #PB_Window_SizeGadget|#PB_Window_ScreenCentered, Title, WindowID)
  If iWnd = #Null
    ProcedureReturn Result
  EndIf
  
  TEXTBOX\hWnd = WindowID(iWnd)
  
  If CreateGadgetList(TEXTBOX\hWnd) = #Null
    ProcedureReturn Result
  EndIf
  
  TEXTBOX\MinWidth  = WindowWidth()
  TEXTBOX\MinHeight = WindowHeight()
  
  TEXTBOX\iFrame    = Frame3DGadget(#PB_Any,  5,  5,   0, 36, "")
  TEXTBOX\iMessage  = TextGadget   (#PB_Any, 10, 17,   0, 20, Message, #PB_Text_Center)
  TEXTBOX\iText     = EditorGadget (#PB_Any,  5, 50,   0,  0)
  TEXTBOX\iOK       = ButtonGadget (#PB_Any,  0,  0, 100, 25, #Req_Ok, #PB_Button_Default)
  
  SetGadgetText(TEXTBOX\iText,Text)
  SetGadgetFont(TEXTBOX\iMessage,FontBold)
  SetGadgetFont(TEXTBOX\iOK,FontBold)
  
  SetWindowCallback(@TextBox_CB())
  ResizeWindow(TEXTBOX\MinWidth,TEXTBOX\MinHeight-10)
  AddKeyboardShortcut(iWnd, #PB_Shortcut_Escape, 0) 
  AddKeyboardShortcut(iWnd, #PB_Shortcut_Return, 1) 
  
  Repeat
    
    Event = WaitWindowEvent()
    
    If EventWindowID() = iWnd
      
      Select Event
        
        Case #PB_Event_CloseWindow : Break
          
        Case #PB_Event_Menu
          
          Select EventMenuID()
            Case 0 : Break
            Case 1 : Break
          EndSelect
          
        Case #PB_Event_Gadget
          
          Select EventGadgetID()
            Case TEXTBOX\iOK : Break
          EndSelect
          
      EndSelect
      
    EndIf
    
  ForEver
  
  FreeGadget(TEXTBOX\iFrame)
  FreeGadget(TEXTBOX\iMessage)
  FreeGadget(TEXTBOX\iText)
  FreeGadget(TEXTBOX\iOK)
  CloseWindow(iWnd)
  
  ProcedureReturn #Null
  
EndProcedure
ProcedureDLL.s TextRequester(WindowID.l, Title.s, Message.s, Text.s)
  
  Result.s = ""
  
  iWnd.l = OpenWindow(#PB_Any, 0, 0, 250, 300, #PB_Window_SizeGadget|#PB_Window_ScreenCentered, Title, WindowID)
  If iWnd = #Null
    ProcedureReturn Result
  EndIf
  
  TEXTREQ\hWnd = WindowID(iWnd)
  
  If CreateGadgetList(TEXTREQ\hWnd) = #Null
    ProcedureReturn Result
  EndIf
  
  TEXTREQ\MinWidth  = WindowWidth()
  TEXTREQ\MinHeight = WindowHeight()
  
  TEXTREQ\iFrame    = Frame3DGadget(#PB_Any,  5,  5,   0, 36, "")
  TEXTREQ\iMessage  = TextGadget   (#PB_Any, 10, 17,   0, 20, Message, #PB_Text_Center)
  TEXTREQ\iText     = EditorGadget (#PB_Any,  5, 50,   0,  0)
  TEXTREQ\iOK       = ButtonGadget (#PB_Any,  5,  0, 100, 25, #Req_Ok, #PB_Button_Default)
  TEXTREQ\iCancel   = ButtonGadget (#PB_Any,  0,  0, 100, 25, #Req_Cancel)
  
  EditorGadgetWrap(GadgetID(TEXTREQ\iText),#True)
  SetGadgetText(TEXTREQ\iText,Text)
  
  SetGadgetFont(TEXTREQ\iMessage,FontBold)
  SetGadgetFont(TEXTREQ\iOK,FontBold)
  SetGadgetFont(TEXTREQ\iCancel,FontBold)
  
  SetWindowCallback(@TextRequester_CB())
  ResizeWindow(TEXTREQ\MinWidth,TEXTREQ\MinHeight-10)
  AddKeyboardShortcut(iWnd, #PB_Shortcut_Escape, 0) 
  
  Repeat
    
    Event = WaitWindowEvent()
    
    If EventWindowID() = iWnd
      
      Select Event
        
        Case #PB_Event_CloseWindow : Break
          
        Case #PB_Event_Menu
          
          Select EventMenuID()
            Case 0 : Break
          EndSelect
          
        Case #PB_Event_Gadget
          
          Select EventGadgetID()
            Case TEXTREQ\iOK : Result = GetGadgetText(TEXTREQ\iText) : Break
            Case TEXTREQ\iCancel : Break    
          EndSelect
          
      EndSelect
      
    EndIf
    
  ForEver
  
  FreeGadget(TEXTREQ\iFrame)
  FreeGadget(TEXTREQ\iMessage)
  FreeGadget(TEXTREQ\iText)
  FreeGadget(TEXTREQ\iOK)
  FreeGadget(TEXTREQ\iCancel)
  CloseWindow(iWnd)
  
  ProcedureReturn Result
  
EndProcedure


;-------------------------------------------------------------------------------
; End of library
;-------------------------------------------------------------------------------


ProcedureDLL RequesterPlus_End()
  
  ; ...
  
EndProcedure
; ExecutableFormat=Windows
; EOF